#region Copyright Syncfusion Inc. 2001 - 2015
// Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics;
using System.Linq;
using System.Text.RegularExpressions;
using Syncfusion.Data.Extensions;
using System.Reflection;

namespace ValidateTest.DataModel
{
    public class UserInfo : INotifyPropertyChanged, INotifyDataErrorInfo, IEditableObject
    {

		public UserInfo()
		{

		}

        public Action<string> MethodCall;
        public Dictionary<string, object> BackUp()
        {
            var dict = new Dictionary<string, object>();
            var itemProperties = this.GetType().GetTypeInfo().DeclaredProperties;
            foreach (var pDescriptor in itemProperties)
            {
                if (pDescriptor.CanWrite)
                    dict.Add(pDescriptor.Name, pDescriptor.GetValue(this));
            }
            return dict;
        }

        #region Fields
        private int _userId;
        private string _name;
        private DateTime _dateofBirth;
        private string _email;
        private string _contactNo;
        private City _city;
        private int _salary;
        private bool _boolTest;
        readonly Regex _emailRegex = new Regex(@"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$");
        #endregion

        #region Properties

        public int UserId
        {
            get { return _userId; }
            set { _userId = value; OnPropertyChanged("UserId"); }
        }

        public string Name
        {
            get { return _name; }
            set { _name = value; OnPropertyChanged("Name"); }
        }

        public DateTime DateofBirth
        {
            get { return _dateofBirth; }
            set { _dateofBirth = value; OnPropertyChanged("DateofBirth"); }
        }

        public string EMail
        {
            get { return _email; }
            set { _email = value; OnPropertyChanged("EMail"); }
        }

        [StringLength(14, ErrorMessage = "The “ContactNo” field must be a string with a maximum length of 14.")]
        public string ContactNo
        {
            get { return _contactNo; }
            set { _contactNo = value; OnPropertyChanged("ContactNo"); }
        }

        public City MyCity
        {
            get { return _city; }
            set { _city = value; OnPropertyChanged("City"); }
        }

		private ObservableCollection<City> m_MultipleCities;
		public ObservableCollection<City> MultipleCities
		{
			get { return m_MultipleCities; }
			set
			{
				m_MultipleCities = value;
				this.OnPropertyChanged("MultipleCities");
			}
		}

        [Range(10000, 30000, ErrorMessage = "The “Salary” field can range from 10000 through 30000.")]
        public int Salary
        {
            get { return _salary; }
            set { _salary = value; OnPropertyChanged("Salary"); }
        }

        public bool BoolTest
        {
            get { return _boolTest; }
            set { _boolTest = value; OnPropertyChanged("BoolTest"); }
        }

        #endregion

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion

        #region INotifyDataErrorInfo
        event EventHandler<DataErrorsChangedEventArgs> INotifyDataErrorInfo.ErrorsChanged
        {
            add { }
            remove { }
        }

        System.Collections.IEnumerable INotifyDataErrorInfo.GetErrors(string propertyName)
        {
            if (propertyName == "EMail")
            {
                if (!_emailRegex.IsMatch(EMail))
                {
                    var errorList = new List<string>();
                    errorList.Add("E-Mail Id is in InValid format!");
                    return errorList;
                }
            }
            return null;
        }

        bool INotifyDataErrorInfo.HasErrors
        {
            get
            {
                return false;
            }
        }
        #endregion


        public Dictionary<string, object> storedValues;
        public void BeginEdit()
        {

            if (this.storedValues != null)
                return;
            this.storedValues = this.BackUp();
            if (MethodCall != null)
                MethodCall("Begin Edit Called");
        }

        public void CancelEdit()
        {

            if (MethodCall != null)
                MethodCall("Cancel Edit Called");
            if (this.storedValues == null)
                return;
            int i = 0;
            foreach (var item in this.storedValues)
            {
                var itemProperties = this.GetType().GetTypeInfo().DeclaredProperties;
                var pDesc = itemProperties.FirstOrDefault(p => p.Name == item.Key);
                if (pDesc != null)
                    pDesc.SetValue(this, item.Value);
                i++;
            }
        }

        public void EndEdit()
        {
            if (MethodCall != null)
                MethodCall("End Edit Called");
            if (this.storedValues != null)
            {
                this.storedValues.Clear();
                this.storedValues = null;
            }

            Debug.WriteLine("End Edit Called");
        }
    }
}